/*
 *	Quarticurve KWindowSystem client
 *
 *	Copyright (C) 1999, 2001 Daniel Duley <mosfet@kde.org>
 *	Matthias Ettrich <ettrich@kde.org>
 *	Karol Szwed <gallium@kde.org>
 *  Than Ngo than@redhat.com
 *  Copyright (C) 2007 Kevin Kofler <Kevin@tigcc.ticalc.org> (KDE 4 port)
 *
 *	Draws mini titlebars for tool windows.
 *	Many features are now customizable.
 */

#ifndef _BLUECURVE_H
#define _BLUECURVE_H

#include <Q3Button>
#include <qbitmap.h>
#include <qdatetime.h>
//Added by qt3to4:
#include <Q3HBoxLayout>
#include <QShowEvent>
#include <QPaintEvent>
#include <QResizeEvent>
#include <Q3GridLayout>
#include <QMouseEvent>
#include <QEvent>
#include <kdecoration.h>
#include <kdecorationfactory.h>


class QSpacerItem;
class Q3BoxLayout;
class Q3GridLayout;
class Q3HBoxLayout;

namespace Quarticurve {

class QuarticurveClient;

class QuarticurveHandler: public KDecorationFactory
{
	public:
		QuarticurveHandler();
		~QuarticurveHandler();
		KDecoration* createDecoration( KDecorationBridge* );
		bool reset(unsigned long changed);
		//virtual QValueList< BorderSize > borderSizes() const;

	private:
		void readConfig();
		void createPixmaps();
		void freePixmaps();
		void drawButtonBackground(QPixmap *pix, 
			const QColorGroup &g,
			bool sunken,
			bool active);
		void recolor( QImage &img, const QColor& color );
		bool supports( Ability ability ) const;
};

enum ButtonPos { ButtonLeft = 0, ButtonMid, ButtonRight, LeftButtonRight };

class QuarticurveButton : public Q3Button, public KDecorationDefines
{
	public:
		QuarticurveButton( QuarticurveClient *parent=0, const char *name=0,
			bool largeButton=true, int pos=ButtonMid,
			bool isOnAllDesktopsButton=false, const unsigned char *bitmap=NULL,
			const QString& tip=NULL, const int realizeBtns=Qt::LeftButton );
		~QuarticurveButton(); 

		int last_button;
		void turnOn( bool isOn );
		void setBitmap(const unsigned char *bitmap);
		void setTipText(const QString &tip);
		QSize sizeHint() const;
		void reset();
		int pos;

	protected:
		void resizeEvent( QResizeEvent* e);
		void showEvent(QShowEvent *ev);
		void doShape();
		void enterEvent(QEvent *);
		void leaveEvent(QEvent *);
		void mousePressEvent( QMouseEvent* e );
		void mouseReleaseEvent( QMouseEvent* e );
		void drawButton(QPainter *p);
		void drawButtonLabel(QPainter*) {;}

		QBitmap* deco;
		bool large;
		bool isLeft;
		bool isOnAllDesktops;
		bool isMouseOver;
		QuarticurveClient* client;

		int realizeButtons;
};


class QuarticurveClient : public KDecoration
{
	Q_OBJECT

	public:
		QuarticurveClient( KDecorationBridge* bridge, KDecorationFactory* factory );
		~QuarticurveClient() {;}

		virtual void init();

	protected:
		virtual void resizeEvent( QResizeEvent* );
		virtual void paintEvent( QPaintEvent* );
		virtual void showEvent( QShowEvent* );
		virtual void mouseDoubleClickEvent( QMouseEvent * );

		virtual void doShape();
		virtual void borders( int&, int&, int&, int& ) const;
		virtual void resize(const QSize&);
		virtual void captionChange();
		virtual void maximizeChange();
		virtual void shadeChange();
		virtual bool isTool() const;

		virtual void activeChange();
		virtual void iconChange();
		virtual void desktopChange();
		virtual QSize minimumSize() const;
		virtual Position mousePosition(const QPoint &) const;

	protected slots:
		void slotMaximize();
		void menuButtonPressed();
		void menuButtonReleased();

	private:
		bool eventFilter( QObject* o, QEvent* e );
		void calcHiddenButtons();
		void addClientButtons( const QString& s, bool isLeft=true );

		enum Buttons{ BtnHelp=0, BtnMax, BtnIconify, BtnClose,
			BtnMenu, BtnOnAllDesktops, BtnCount };
		QuarticurveButton* button[ QuarticurveClient::BtnCount ];

		int           lastButtonWidth;
		int           titleHeight;
		bool          largeButtons;
		Q3GridLayout*  g;
		Q3HBoxLayout*  hb;
		QSpacerItem*  titlebar;
		QSpacerItem*  spacer;
		bool          m_closing;
};

}

#endif
// vim: ts=4
